// Generated by CoffeeScript 1.10.0
(function() {
  var KicadGenerator, fs, mkdirp, sprintf;

  fs = require('fs');

  mkdirp = require('mkdirp');

  sprintf = require('sprintf-js').sprintf;

  KicadGenerator = (function() {
    function KicadGenerator(library) {
      this.library = library;
      this.f = "%." + this.library.pattern.decimals + "f";
    }

    KicadGenerator.prototype.generate = function(name) {
      var dir, element, fd, i, j, len, len1, now, pattern, patternName, patterns, ref, ref1, results, timestamp;
      dir = './kikad';
      mkdirp.sync(dir + "/" + name + ".pretty");
      patterns = {};
      now = new Date;
      timestamp = (now.getDate()) + "/" + (now.getMonth() + 1) + "/" + (now.getYear() + 1900) + " " + (now.getHours()) + ":" + (now.getMinutes()) + ":" + (now.getSeconds());
      fd = fs.openSync(dir + "/" + name + ".lib", 'w');
      fs.writeSync(fd, "EESchema-LIBRARY Version 2.3 Date: " + timestamp + "\n");
      fs.writeSync(fd, '#encoding utf-8\n');
      ref = this.library.elements;
      for (i = 0, len = ref.length; i < len; i++) {
        element = ref[i];
        this._generateSymbol(fd, element);
        ref1 = element.patterns;
        for (j = 0, len1 = ref1.length; j < len1; j++) {
          pattern = ref1[j];
          patterns[pattern.name] = pattern;
        }
      }
      fs.writeSync(fd, '# End Library\n');
      fs.closeSync(fd);
      console.log("Generating KiCad library '" + name + ".lib': OK");
      results = [];
      for (patternName in patterns) {
        pattern = patterns[patternName];
        fd = fs.openSync(dir + "/" + name + ".pretty/" + patternName + ".kicad_mod", 'w');
        this._generatePattern(fd, pattern);
        console.log("Generating KiCad footprint '" + patternName + ".kicad_mod': OK");
        results.push(fs.closeSync(fd));
      }
      return results;
    };

    KicadGenerator.prototype._generatePattern = function(fd, pattern) {
      var i, len, patObj, ref, shape;
      fs.writeSync(fd, "(module " + pattern.name + " (layer F.Cu)\n");
      fs.writeSync(fd, "  (attr smd)\n");
      ref = pattern.shapes;
      for (i = 0, len = ref.length; i < len; i++) {
        shape = ref[i];
        patObj = this._patternObj(shape);
        switch (patObj.kind) {
          case 'circle':
            fs.writeSync(fd, sprintf("  (fp_circle (center " + this.f + " " + this.f + ") (end " + this.f + " " + this.f + ") (layer %s) (width " + this.f + "))\n", patObj.x, patObj.y, patObj.x, patObj.y + patObj.radius, patObj.layer, patObj.lineWidth));
            break;
          case 'line':
            fs.writeSync(fd, sprintf("  (fp_line (start " + this.f + " " + this.f + ") (end " + this.f + " " + this.f + ") (layer %s) (width " + this.f + "))\n", patObj.x1, patObj.y1, patObj.x2, patObj.y2, patObj.layer, patObj.lineWidth));
            break;
          case 'pad':
            fs.writeSync(fd, sprintf("  (pad %s %s %s (at " + this.f + " " + this.f + ") (size " + this.f + " " + this.f + ") (layers %s))\n", patObj.name, patObj.type, patObj.shape, patObj.x, patObj.y, patObj.width, patObj.height, patObj.layer));
        }
      }
      return fs.writeSync(fd, ")\n");
    };

    KicadGenerator.prototype._generateSymbol = function(fd, element) {
      var i, len, nameObj, pinNameSpace, ref, ref1, ref2, refObj, shape, showPinNames, showPinNumbers, symObj, symbol;
      symbol = element.symbol;
      symbol.invertVertical();
      refObj = this._symbolObj(symbol.attributes['refDes']);
      nameObj = this._symbolObj(symbol.attributes['name']);
      fs.writeSync(fd, "#\n# " + element.name + "\n#\n");
      showPinNumbers = ((ref = element.schematics) != null ? ref.showPinNumbers : void 0) ? 'Y' : 'N';
      showPinNames = ((ref1 = element.schematics) != null ? ref1.showPinNames : void 0) ? 'Y' : 'N';
      pinNameSpace = Math.round(this.library.symbol.space.pinName);
      fs.writeSync(fd, "DEF " + element.name + " " + element.refDes + " 0 " + pinNameSpace + " " + showPinNumbers + " " + showPinNames + " 1 L N\n");
      fs.writeSync(fd, "F0 \"" + element.refDes + "\" " + refObj.x + " " + refObj.y + " " + refObj.fontSize + " H V " + refObj.halign + " " + refObj.valign + "NN\n");
      fs.writeSync(fd, "F1 \"" + element.name + "\" " + nameObj.x + " " + nameObj.y + " " + nameObj.fontSize + " H V " + nameObj.halign + " " + nameObj.valign + "NN\n");
      fs.writeSync(fd, "DRAW\n");
      ref2 = element.symbol.shapes;
      for (i = 0, len = ref2.length; i < len; i++) {
        shape = ref2[i];
        symObj = this._symbolObj(shape);
        switch (symObj.kind) {
          case 'pin':
            fs.writeSync(fd, "X " + symObj.name + " " + symObj.number + " " + symObj.x + " " + symObj.y + " " + symObj.length + " " + symObj.orientation + " " + symObj.fontSizeNum + " " + symObj.fontSizeName + " 1 1 " + symObj.type + symObj.shape + "\n");
            break;
          case 'rectangle':
            fs.writeSync(fd, "S " + symObj.x + " " + symObj.y + " " + (symObj.x + symObj.width) + " " + (symObj.y + symObj.height) + " 1 1 0 " + symObj.fillStyle + "\n");
        }
      }
      fs.writeSync(fd, "ENDDRAW\n");
      return fs.writeSync(fd, "ENDDEF\n");
    };

    KicadGenerator.prototype._patternObj = function(shape) {
      var obj;
      obj = shape;
      if (obj.shape === 'rectangle') {
        obj.shape = 'rect';
      }
      obj.layer = (function() {
        switch (obj.layer) {
          case 'top':
            if (obj.kind === 'pad') {
              return 'F.Cu F.Paste F.Mask';
            } else {
              return 'F.Cu';
            }
            break;
          case 'topSilkscreen':
            return 'F.SilkS';
          case 'topAssembly':
            return 'F.Fab';
          case 'bottom':
            if (obj.kind === 'pad') {
              return 'B.Cu B.Paste B.Mask';
            } else {
              return 'B.Cu';
            }
            break;
          case 'bottomSilkscreen':
            return 'B.SilkS';
          case 'bottomAssembly':
            return 'B.Fab';
          default:
            return 'F.Cu';
        }
      })();
      return obj;
    };

    KicadGenerator.prototype._symbolObj = function(shape) {
      var obj;
      obj = shape;
      obj.x = Math.round(obj.x);
      obj.y = Math.round(obj.y);
      obj.length = Math.round(obj.length);
      if (obj.halign != null) {
        obj.halign = (function() {
          switch (obj.halign) {
            case 'center':
              return 'C';
            case 'right':
              return 'R';
            default:
              return 'L';
          }
        })();
      }
      if (obj.valign != null) {
        obj.valign = (function() {
          switch (obj.valign) {
            case 'center':
              return 'C';
            case 'bottom':
              return 'B';
            default:
              return 'T';
          }
        })();
      }
      if (obj.orientation != null) {
        obj.orientation = (function() {
          switch (obj.orientation) {
            case 'left':
              return 'L';
            case 'up':
              return 'U';
            case 'down':
              return 'D';
            default:
              return 'R';
          }
        })();
      }
      obj.fontSize = Math.round(this.library.symbol.fontSize[shape.name] != null ? this.library.symbol.fontSize[shape.name] : this.library.symbol.fontSize["default"]);
      obj.fontSizeNum = Math.round(this.library.symbol.fontSize.pinNumber);
      obj.fontSizeName = Math.round(this.library.symbol.fontSize.pinName);
      obj.type = 'U';
      if (obj.power || obj.ground) {
        obj.type = 'W';
        if (obj.output) {
          obj.type = 'w';
        }
      } else {
        if (obj.input && obj.output) {
          obj.type = 'B';
        } else if (obj.input) {
          obj.type = 'I';
        } else {
          obj.type = 'O';
        }
        if (obj.z) {
          obj.type = 'T';
        }
      }
      obj.shape = '';
      if (obj.invisible) {
        obj.shape += 'N';
      }
      if (obj.inverted) {
        obj.shape += 'I';
      }
      if (obj.shape !== '') {
        obj.shape = ' ' + obj.shape;
      }
      switch (obj.fill) {
        case 'foreground':
          obj.fillStyle = 'f';
          break;
        case 'backgroung':
          obj.fillStyle = 'F';
          break;
        default:
          obj.fillStyle = 'N';
      }
      return obj;
    };

    return KicadGenerator;

  })();

  module.exports = KicadGenerator;

}).call(this);
